#ifndef _EZNRAPI_H_
#define _EZNRAPI_H_

#include "ezsmcard.h"


#ifdef __cplusplus
extern "C" {
#endif


//
// for ReaderType (LONG : 0 - 31 bits)
//     Product : 31 - 24
//     Interface : 23 - 16
//	   Version : 15 - 8
//
#define EZ100N_PRODUCT		0x01000000 
#define EZ200N_PRODUCT		0x02000000
#define RS232_INTERFACE		0x00010000
#define PS2_INTERFACE		0x00020000
#define USB_INTERFACE		0x00040000 

//
// PortNumber for EzConnectReader
//
#define COM1	0
#define COM2	1
#define COM3	2
#define COM4	3


#define PS2_PORT	4
#define USB1		5
#define USB2		6
#define USB3		7
#define USB4		8
///////////////////////////////////////////

//
// the flag for SelectCard used in EzConnectCard, EzDisconnectCard
//
#define MAIN_CARD 0

// not support yet
#define SAM_CARD  1

#define DEFAULT_CARD MAIN_CARD

//
// Card Type
//
#define UNKNOWN_CARD	0
#define CPU_CARD		1
#define MEMORY_CARD		2


//
// Card State
//
#define EZ_CARD_ABSENT		0
#define EZ_CARD_PRESENT		1
#define EZ_CARD_POWERED		2


//
// Constant
//
#define MAX_ATR_SIZE		64
#define EZ_READER_TYPE_STR_LEN	17
#define DEFAULT_APDU_IN_SIZE	256
#define DEFAULT_APDU_OUT_SIZE	256

//
//	ATR Structure
//
typedef struct{

		BYTE    CardType;

		struct{

			UCHAR   Length;

			UCHAR   Buffer[MAX_ATR_SIZE];

		}ATR;		


}EZ_ATR, *PEZ_ATR;

//
//  APDU structure
//
typedef struct{

		BYTE    CLA;

		BYTE    INS;

		BYTE    P1;

		BYTE    P2;		

		union{

			// for T1
			struct{						

				BYTE b0;
						
				BYTE b1;

				BYTE b2;

				//	if the protocol is T1, "Used" indicates that the number of Lc bytes will
				//		be sent. The range is between 0 and 3;
				BYTE Used;

			}Lc;
					
			// for T0
			BYTE P3;

		};		
		
				
		// for T1								
		struct{
			
			BYTE b0;

			BYTE b1;

			BYTE b2;

			//	if the protocol is T1, "Used" indicates that the number of Le bytes will
			//		be sent. The range is between 0 and 3;
			BYTE Used;

		}Le;					

		struct{

			// the size of the DataIn buffer			
			DWORD	BufferSize;

			// the length of the data in this buffer to be sent
			DWORD	BufferLength;

			// data buffer to send
			PBYTE	Buffer;

		}DataIn;		

		struct{

			// the size of the DataOut buffer			
			DWORD	BufferSize;

			// the length of the data that is returned from the ICC.
			DWORD	BufferLength;

			// data buffer to receive
			PBYTE	Buffer;

		}DataOut;		
			
		DWORD	 Status;

}EZ_APDU, *PEZ_APDU;


/******************************************************************************************
//  
//	EZ-READER N-SERIES SMARTCARD APPLICATION INTERFACE
//
******************************************************************************************/

// Open the Reader
extern EZSMC_STATUS WINAPI
EzConnectReader(
	IN	ULONG		PortNumber,   
	OUT PEZHANDLE   pHandle,
    OUT	PULONG      pReaderType
);

// Close the Reader
extern EZSMC_STATUS WINAPI
EzDisconnectReader(
	IN	EZHANDLE	Handle
);

// Power On the card and Set Protocol
extern EZSMC_STATUS WINAPI
EzConnectCard(
	IN	EZHANDLE	Handle,
	IN	ULONG		SelectCard,
	IN	ULONG		PreferedProtocols,
    OUT	PULONG		pActiveProtocol,
	OUT	PEZ_ATR		pATR
);


extern EZSMC_STATUS WINAPI
EzDisconnectCard(
	IN	EZHANDLE	Handle,
	IN	ULONG		SelectCard
);

extern EZSMC_STATUS WINAPI
EzAPDUInit(
    IN OUT PEZ_APDU	pAPDU,
	IN DWORD		DataInBufferSize,
	IN DWORD		DataOutBufferSize
);

extern EZSMC_STATUS WINAPI
EzSendAPDU(
	IN EZHANDLE		Handle,
	IN ULONG		SelectCard,
	IN OUT PEZ_APDU	pAPDU
);

//
// Transmit APDU to CPU card and receive data from it. 
// (with using two simple array buffers)
//
extern EZSMC_STATUS WINAPI
EzTransmit(
	IN EZHANDLE		Handle,
	IN ULONG		SelectCard,
	IN LPBYTE		pbSendBuffer,
	IN DWORD		SendLength,
	OUT LPBYTE		pbRecvBuffer,
	IN OUT LPDWORD	pRecvLength
);

extern EZSMC_STATUS WINAPI
EzGetState(
	IN EZHANDLE		Handle,
	IN ULONG		SelectCard,
	OUT PULONG		pState
);


extern EZSMC_STATUS WINAPI
EzGetReaderName(
	IN EZHANDLE		Handle,
	OUT PUCHAR		RecvBuffer,
	IN ULONG		RecvBufferSize,
	OUT ULONG*		RecvBufferLength
);

extern EZSMC_STATUS WINAPI
EzWaitCardInsert(
	IN EZHANDLE		Handle,
	IN ULONG		SelectCard,
	IN ULONG		WaitTime
);


extern EZSMC_STATUS WINAPI
EzWaitCardRemove(
	IN EZHANDLE		Handle,
	IN ULONG		SelectCard,
	IN ULONG		WaitTime
);



/***********************************************************************
//
//   EZ-READER N-SERIES Memory Card Application Interface
//
***********************************************************************/

#define MM_ADDRESS ULONG

////////////////////////////////////////////////////////////////////////
//  SLE4432/4442 Memory Card  API
////////////////////////////////////////////////////////////////////////

extern EZSMC_STATUS WINAPI
SLE4442_Read_Main_Memory(
    IN EZHANDLE hCard,
	OUT LPBYTE pbRecvBuffer,
	IN MM_ADDRESS aStartAddr,
	IN OUT LPDWORD pcbRecvLength);

extern EZSMC_STATUS WINAPI
SLE4442_Update_Main_Memory(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4442_Update_Main_MemoryA(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4442_Read_Protection_Memory(
    IN EZHANDLE hCard,
	OUT LPBYTE pbRecvBuffer);

extern EZSMC_STATUS WINAPI
SLE4442_Write_Protection_Memory(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
	IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);


////////////////////////////////////////////////////////////////////////
//  SLE4442 Memory Card  API
////////////////////////////////////////////////////////////////////////

extern EZSMC_STATUS WINAPI
SLE4442_Read_Security_Memory(
    IN EZHANDLE hCard,
	OUT LPBYTE pbRecvBuffer);

extern EZSMC_STATUS WINAPI
SLE4442_Update_Security_Memory(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
	IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4442_Compare_Verification_Data(
    IN EZHANDLE hCard,
	IN BYTE PSC1,
	IN BYTE PSC2,
	IN BYTE PSC3);


////////////////////////////////////////////////////////////////////////
//  SLE4418/4428 Memory Card  API
////////////////////////////////////////////////////////////////////////

extern EZSMC_STATUS WINAPI
SLE4428_Write_Data_With_Protect_Bit(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength); 

extern EZSMC_STATUS WINAPI
SLE4428_Write_Data_With_Protect_BitA(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);
   
extern EZSMC_STATUS WINAPI
SLE4428_Write_Data_Without_Protect_Bit(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4428_Write_Data_Without_Protect_BitA(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4428_Write_Protect_Bit(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
	IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4428_Read_Data_With_Protect_Bit(
    IN EZHANDLE hCard,
	OUT LPBYTE pbRecvBuffer,
	OUT LPBYTE pbProtectBuffer,
	IN MM_ADDRESS aStartAddr,
	IN OUT LPDWORD pcbRecvLength);

extern EZSMC_STATUS WINAPI
SLE4428_Read_Data_Without_Protect_Bit(
    IN EZHANDLE hCard,
	OUT LPBYTE pbRecvBuffer,	
	IN MM_ADDRESS aStartAddr,
	IN OUT LPDWORD pcbRecvLength);

////////////////////////////////////////////////////////////////////////
//  SLE4428 Memory Card  API
////////////////////////////////////////////////////////////////////////
extern EZSMC_STATUS WINAPI
SLE4428_PSC_Verification(
    IN EZHANDLE hCard,
	IN BYTE PSC1,
	IN BYTE PSC2);

////////////////////////////////////////////////////////////////////////
//  SLE4404 Memory Card  API
////////////////////////////////////////////////////////////////////////

extern EZSMC_STATUS WINAPI
SLE4404_Read_Memory(
    IN EZHANDLE hCard,
	OUT LPBYTE pbRecvBuffer,
	IN MM_ADDRESS aStartAddr,
	IN OUT LPDWORD pcbRecvLength);

extern EZSMC_STATUS WINAPI
SLE4404_Read_User_Memory(
    IN EZHANDLE hCard,
	OUT LPBYTE pbRecvBuffer,
	IN MM_ADDRESS aStartAddr,
	IN OUT LPDWORD pcbRecvLength);

extern EZSMC_STATUS WINAPI
SLE4404_Compare_User_Code(
    IN EZHANDLE hCard,
	IN BYTE Code1,
	IN BYTE Code2);

extern EZSMC_STATUS WINAPI
SLE4404_Compare_Memory_Code(
    IN EZHANDLE hCard,
	IN BYTE Code1,
	IN BYTE Code2,
	IN BYTE Code3,
	IN BYTE Code4);

extern EZSMC_STATUS WINAPI
SLE4404_Erase_Memory(
    IN EZHANDLE hCard,	
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4404_Write_Memory(
    IN EZHANDLE hCard,
 	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4404_Write_MemoryA(
    IN EZHANDLE hCard,
 	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4404_Write_User_Memory(
    IN EZHANDLE hCard,
 	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4404_Write_User_MemoryA(
    IN EZHANDLE hCard,
 	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4404_Enter_Test_Mode(
	IN EZHANDLE hCard);

extern EZSMC_STATUS WINAPI
SLE4404_Exit_Test_Mode(
	IN EZHANDLE hCard);

extern EZSMC_STATUS WINAPI
SLE4404_Blow_Fuse(
    IN EZHANDLE hCard);



////////////////////////////////////////////////////////////////////////
//  SLE4406/4436/5536 Memory Card API
////////////////////////////////////////////////////////////////////////
extern EZSMC_STATUS WINAPI
SLE4436_Read_Memory(
    IN  EZHANDLE hCard,
	OUT LPBYTE pbRecvBuffer,
	IN  MM_ADDRESS aStartAddr,
	IN  OUT LPDWORD pcbRecvLength);

extern EZSMC_STATUS WINAPI
SLE4436_Read_Counter_Stages(
    IN  EZHANDLE hCard,
	OUT BYTE Stage[5]);

extern EZSMC_STATUS WINAPI
SLE4436_Write_Memory(
    IN EZHANDLE hCard,
	IN LPBYTE pbTransmitBuffer,
    IN MM_ADDRESS aStartAddr,
	IN DWORD TransmitLength);

extern EZSMC_STATUS WINAPI
SLE4436_Write_Counter_Stage(
    IN EZHANDLE hCard,	
	IN BYTE Stage,
	IN BYTE Data);

extern EZSMC_STATUS WINAPI
SLE4436_Reload(
    IN EZHANDLE hCard,
    IN BYTE BitAddr);

extern EZSMC_STATUS WINAPI
SLE4436_Verify_Transport_Code(
    IN EZHANDLE hCard,
	IN BYTE Code1,
	IN BYTE Code2,
	IN BYTE Code3);

////////////////////////////////////////////////////////////////////////
//  SLE4436/5536 Memory Card API
////////////////////////////////////////////////////////////////////////
extern EZSMC_STATUS WINAPI
SLE4436_Authentication(
    IN EZHANDLE hCard,
	OUT BYTE ReturnCode[2],	
	IN BYTE Key,
	IN BYTE ClockPulse,
	IN BYTE Code[6]);

////////////////////////////////////////////////////////////////////////
//  SLE5536 Memory Card API
////////////////////////////////////////////////////////////////////////
/* This function will be added on next version
extern EZSMC_STATUS WINAPI
SLE5536_Extended_Authentication(
    IN EZHANDLE hCard,
	OUT BYTE ReturnCode[2],	
	IN BYTE Key,
	IN BYTE ClockPulse,
	IN BYTE Code[6]);
*/
////////////////////////////////////////////////////////////////////////
//  I2C Memory Card  API
////////////////////////////////////////////////////////////////////////

extern EZSMC_STATUS WINAPI
I2C_Read_Memory(
    IN EZHANDLE hCard,    
	IN BYTE ReadCommand, 
	IN MM_ADDRESS aStartAddr,
	OUT LPBYTE pbRecvBuffer,
	IN OUT LPDWORD pcbRecvLength,
	IN BYTE Dummy,
	IN BYTE AddrByteNum);

extern EZSMC_STATUS WINAPI
I2C_Write_Memory(
    IN EZHANDLE hCard,    
	IN BYTE WriteCommand, 
	IN MM_ADDRESS aStartAddr,
	OUT LPBYTE pbTransmitBuffer,
	IN DWORD TransmitLength,
	IN BYTE AddrByteNum);


////////////////////////////////////////////////////////////////////////
extern EZSMC_STATUS WINAPI
EzSetC8PinToHigh(
    IN EZHANDLE hCard);

extern EZSMC_STATUS WINAPI
EzSetC8PinToLow(			
    IN EZHANDLE hCard);


////////////////////////////////////////////////////////////////////////




#ifdef __cplusplus
}
#endif



#endif